package com.flow.common.core.model;

import lombok.Data;
import org.springframework.security.core.CredentialsContainer;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.Set;
@Data
public class OauthUserDetails implements UserDetails, CredentialsContainer {
    private static final long serialVersionUID = 530L;
    private  String nickName;
    private  String leader;
    private Set<String> roleIds;
    private Long deptId;
    private String phone;
    private String email;
    private String password;
    private final String username;
    private final boolean enabled;
    private final Set<GrantedAuthority> authorities;

    public OauthUserDetails(String username,
                            String password,
                            String nickName,
                            String leader,
                            Set<String> roleIds,
                            Long deptId,
                            String phone,
                            String email,
                            boolean enabled,
                            Set<GrantedAuthority> authorities
    ) {
        if (username != null && !username.isEmpty() && password != null) {
            this.username = username;
            this.password = password;
            this.enabled = enabled;
            this.nickName = nickName;
            this.leader = leader;
            this.roleIds = roleIds;
            this.deptId = deptId;
            this.phone = phone;
            this.email = email;
            this.authorities = authorities;
        } else {
            throw new IllegalArgumentException("Cannot pass null or empty values to constructor");
        }
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return this.authorities;
    }

    @Override
    public String getPassword() {
        return this.password;
    }

    @Override
    public String getUsername() {
        return this.username;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return this.enabled;
    }

    @Override
    public void eraseCredentials() {
        this.password = null;
    }

}
