package com.flow.common.core.model;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.flow.common.core.util.ApplicationContextUtil;
import com.flow.common.core.util.StatusCode;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @author caixiaofeng
 * @version 1.0
 * @Description: <<>>
 * @company cxf
 * @create 2021-01-31 22:58
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class Result<T> {
    //是否成功
    private boolean success;
    //状态码
    private Integer code;
    //信息
    private String message;
    //数据
    private T data;

    public Result(boolean success, Integer code, String message) {
        this.success = success;
        this.code = code;
        this.message = message;
    }

    /**
     * 返回成功
     *
     * @return
     */
    public static <T> Result<T> success() {
        Result<T> response = new Result<>();
        response.setSuccess(true);
        response.setCode(StatusCode.SUCCESS);
        response.setMessage("操作成功");
        return response;
    }

    /**
     * 返回成功
     *
     * @return
     */
    public static <T> Result<T> success(String message) {
        Result<T> response = new Result<>();
        response.setSuccess(true);
        response.setCode(StatusCode.SUCCESS);
        response.setMessage(message);
        return response;
    }

    /**
     * 返回成功
     * @param data
     * @return
     * @param <T>
     */
    public static <T> Result<T> successData(T data) {
        Result<T> response = new Result<>();
        response.setSuccess(true);
        response.setData(data);
        response.setCode(StatusCode.SUCCESS);
        return response;
    }

    /**
     * 返回成功
     *
     * @param data 数据
     * @return
     */
    public static <T> Result<T> success(T data) {
        Result<T> response = new Result<>();
        response.setSuccess(true);
        response.setCode(StatusCode.SUCCESS);
        response.setMessage("操作成功");
        response.setData(data);
        return response;
    }

    /**
     * 返回成功
     *
     * @param data 数据
     * @return
     */
    public static <T> Result<T> success(String message, T data) {
        Result<T> response = new Result<>();
        response.setSuccess(true);
        response.setCode(StatusCode.SUCCESS);
        response.setMessage(message);
        response.setData(data);
        return response;
    }

    /**
     * 返回失败
     *
     * @return
     */
    public static <T> Result<T> error() {
        Result<T> response = new Result<>();
        response.setSuccess(false);
        response.setCode(StatusCode.FAILURE);
        response.setMessage("操作失败");
        return response;
    }

    /**
     * 返回失败
     *
     * @param data 数据
     * @return
     */
    public static <T> Result<T> error(T data) {
        Result<T> response = new Result<>();
        response.setSuccess(false);
        response.setCode(StatusCode.FAILURE);
        response.setMessage("操作失败");
        response.setData(data);
        return response;
    }

    /**
     * 返回失败
     *
     * @param message 信息
     * @return
     */
    public static <T> Result<T> error(String message) {
        Result<T> response = new Result<>();
        response.setSuccess(false);
        response.setCode(StatusCode.FAILURE);
        response.setMessage(message);
        return response;
    }

    /**
     * 返回失败
     *
     * @param data 数据
     * @return
     */
    public static <T> Result<T> error(String message, T data) {
        Result<T> response = new Result<>();
        response.setSuccess(false);
        response.setCode(StatusCode.FAILURE);
        response.setMessage(message);
        response.setData(data);
        return response;
    }

    /**
     * 返回失败
     *
     * @param message 信息
     * @return
     */
    public static <T> Result<T> error(String message, Integer code) {
        Result<T> response = new Result<>();
        response.setSuccess(false);
        response.setCode(code);
        response.setMessage(message);
        return response;
    }

    /**
     * 直接响应错误
     *
     * @param code
     * @param message
     * @throws IOException
     */
    public static void sendError(int code, String message) throws IOException {
        HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
        response.setContentType("application/json;charset=UTF-8");
        response.setStatus(code);
        PrintWriter writer = response.getWriter();
        Result result = Result.error(message, code);
        ObjectMapper objectMapper = ApplicationContextUtil.getBean(ObjectMapper.class);
        String json = objectMapper.writeValueAsString(result);
        writer.write(json);
        writer.flush();
        writer.close();
    }

}
