package com.flow.common.core.util;

import com.flow.common.core.model.OauthUserDetails;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public class SecurityContextUtil {

    /**
     * 获得当前登录用户ID
     *
     * @return
     */
    public static String getUserId() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (Objects.nonNull(authentication)) {
            return authentication.getName();
        }
        return null;
    }

    /**
     * 获得当前登录用户昵称
     *
     * @return
     */
    public static String getNickName() {
        OauthUserDetails userDetails = getUserInfo();
        if (Objects.nonNull(userDetails)) {
            return userDetails.getNickName();
        }
        return null;
    }

    /**
     * 登录用户信息
     *
     * @return
     */
    public static OauthUserDetails getUserInfo() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication != null) {
            return (OauthUserDetails) authentication.getPrincipal();
        }
        return null;
    }

    /**
     * 获得当前登录用户Token
     *
     * @return
     */
    public static String getAccessToken() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication != null) {
            OAuth2AuthenticationDetails details = (OAuth2AuthenticationDetails) authentication.getDetails();
            return details.getTokenValue();
        }
        return null;
    }


    /**
     * 获取当前登录用户的权限
     *
     * @return
     */
    public static List<String> getCurrentAuthorities() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication != null) {
            Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();
            return authorities.stream().map(GrantedAuthority::getAuthority).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

}
