package com.flow.flowable.cmd;

import org.flowable.common.engine.impl.interceptor.Command;
import org.flowable.common.engine.impl.interceptor.CommandContext;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.impl.persistence.entity.ExecutionEntity;
import org.flowable.engine.impl.util.CommandContextUtil;
import org.flowable.engine.runtime.Execution;

import java.util.List;

public class GetMultiInstanceRootExecutionCmd implements Command<Execution> {
    protected String processInstanceId;
    protected String taskDefinitionKey;

    public GetMultiInstanceRootExecutionCmd(String processInstanceId, String taskDefinitionKey) {
        this.processInstanceId = processInstanceId;
        this.taskDefinitionKey = taskDefinitionKey;
    }

    @Override
    public Execution execute(CommandContext commandContext) {
        RuntimeService runtimeService = CommandContextUtil.getProcessEngineConfiguration(commandContext).getRuntimeService();
        List<Execution> executions = runtimeService.createExecutionQuery()
                .processInstanceId(processInstanceId)
                .activityId(taskDefinitionKey)
                .list();
        Execution multiInstanceRootExecution = null;
        Execution currentExecution = executions.get(0);
        while (currentExecution != null && multiInstanceRootExecution == null && currentExecution.getParentId() != null) {
            if (((ExecutionEntity) currentExecution).isMultiInstanceRoot()) {
                multiInstanceRootExecution = currentExecution;
            } else {
                currentExecution = runtimeService.createExecutionQuery()
                        .executionId(currentExecution.getParentId()).singleResult();
            }
        }
        return multiInstanceRootExecution;
    }
}
