package com.flow.flowable.cmd;


import com.flow.flowable.utils.ProcessElementUtil;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.common.engine.api.FlowableException;
import org.flowable.common.engine.impl.interceptor.Command;
import org.flowable.common.engine.impl.interceptor.CommandContext;
import org.flowable.editor.language.json.converter.util.CollectionUtils;
import org.flowable.engine.impl.persistence.entity.ExecutionEntity;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityManager;
import org.flowable.engine.impl.util.CommandContextUtil;
import org.flowable.engine.impl.util.ProcessDefinitionUtil;
import org.flowable.engine.runtime.Execution;

import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

public class JumpActivityCmd implements Command<List<String>> {
    protected String processInstanceId;
    protected String activityId;

    public JumpActivityCmd(String processInstanceId, String activityId) {
        this.processInstanceId = processInstanceId;
        this.activityId = activityId;
    }

    @Override
    public List<String> execute(CommandContext commandContext) {
        /*Expression expression = CommandContextUtil.getProcessEngineConfiguration().getExpressionManager().createExpression("conditionExpression");
        Condition condition = new UelExpressionCondition(expression);
        condition.evaluate(sequenceFlow.getId(), "execution");*/
        ExecutionEntityManager executionManager = CommandContextUtil.getExecutionEntityManager(commandContext);
        ExecutionEntity processInstance = executionManager.findById(processInstanceId);
        if (Objects.isNull(processInstance)) {
            throw new FlowableException("流程实例不存在");
        }
        BpmnModel bpmnModel = ProcessDefinitionUtil.getBpmnModel(processInstance.getProcessDefinitionId());
        FlowElement flowElement = bpmnModel.getFlowElement(activityId);
        Set<FlowElement> childElements = ProcessElementUtil.findChildElements(flowElement);
        Set<String> nodeIds = childElements.stream().map(FlowElement::getId).collect(Collectors.toSet());
        // 获取要跳转执行
        List<ExecutionEntity> executions = executionManager.findChildExecutionsByParentExecutionId(processInstanceId);
        List<String> executionIds = executions.stream()
                .map(Execution::getId)
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(nodeIds)) {
            executionIds = executions.stream()
                    .filter(e -> nodeIds.contains(e.getActivityId()))
                    .map(Execution::getId)
                    .collect(Collectors.toList());
        }
        // 跳转执行
        if (CollectionUtils.isNotEmpty(executionIds)) {
            CommandContextUtil.getProcessEngineConfiguration(commandContext)
                    .getRuntimeService()
                    .createChangeActivityStateBuilder()
                    .processInstanceId(processInstanceId)
                    .moveExecutionsToSingleActivityId(executionIds, activityId)
                    .changeState();
        }
        return executionIds;
    }
}
