package com.flow.common.mybatis.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.IService;
import com.flow.common.mybatis.entity.FilterCondition;
import com.flow.common.mybatis.entity.FilterGroup;

import java.util.List;

/**
 * 基础服务接口，所有Service接口都要继承
 *
 * @param <T>
 */
public interface BaseService<T> extends IService<T> {


    default QueryWrapper<T> buildQuery(FilterGroup filterGroup) {
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        buildQuery(queryWrapper, filterGroup);
        List<FilterGroup> groups = filterGroup.getGroups();
        if (CollectionUtils.isNotEmpty(groups)) {
            groups.forEach(group -> buildQuery(queryWrapper, group));
        }
        return queryWrapper;
    }

    default void buildQuery(QueryWrapper<T> queryWrapper, FilterGroup filterGroup) {
        List<FilterCondition> conditions = filterGroup.getConditions();
        if (CollectionUtils.isNotEmpty(conditions)) {
            conditions.forEach(condition -> {
                String field = StringUtils.camelToUnderline(condition.getField());
                String operator = condition.getOperator();
                Object value = condition.getValue();
                switch (operator) {
                    case "eq":
                        queryWrapper.eq(field, value);
                        break;
                    case "ne":
                        queryWrapper.ne(field, value);
                        break;
                    case "gt":
                        queryWrapper.gt(field, value);
                        break;
                }
            });
        }
    }
}