package com.flow.common.oauth2.configure;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.security.servlet.UserDetailsServiceAutoConfiguration;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.access.AccessDeniedHandler;

@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true, proxyTargetClass = true, securedEnabled = true, jsr250Enabled = true)
@EnableAutoConfiguration(exclude = UserDetailsServiceAutoConfiguration.class)
public class ResourceServerConfigure extends ResourceServerConfigurerAdapter {
    @Autowired(required = false)
    private AccessDeniedHandler accessDeniedHandler;
    @Autowired(required = false)
    private AuthenticationEntryPoint authenticationEntryPoint;

    @Override
    public void configure(HttpSecurity http) throws Exception {
        http     //关闭csrf保护
                .csrf().disable()
                //不使用session
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                // 认证请求
                .authorizeRequests()
                //放行预请求
                .antMatchers(HttpMethod.OPTIONS, "/**").permitAll()
                .antMatchers("/static/file/**").permitAll()
                .antMatchers("/auth/login", "/auth/code", "/auth/logout", "/user/list", "/model/download").permitAll()
                // websocket路径放行
                .antMatchers("/socket/**").permitAll()
                // 其他所有请求都需要认证
                .anyRequest().authenticated();
    }

    @Override
    public void configure(ResourceServerSecurityConfigurer resources) {
        // 必须要要AuthorizationServer的资源编号保持一致
        resources.resourceId("flow-app");
        // 身份验证入口点
        if (authenticationEntryPoint != null) {
            resources.authenticationEntryPoint(authenticationEntryPoint);
        }
        // 拒绝访问处理程序
        if (accessDeniedHandler != null) {
            resources.accessDeniedHandler(accessDeniedHandler);
        }
    }
}
