package com.flow.constant;

import java.text.DecimalFormat;

public interface FileConstant {
    /**
     * 文件大小限制10M
     */
    Long MAX_SIZE = 1024 * 1024 * 10L;
    /**
     * windows文件路径
     */
    String WINDOWS = "D:/flow/file/";
    /**
     * linux文件路径
     */
    String LINUX = "/home/flow/file/";
    /**
     * mac文件路径
     */
    String MAC = "/Users/flow/file/";

    /**
     * 文件上传路径
     *
     * @return
     */
    static String getPath() {
        String os = System.getProperty("os.name");
        if (os.toLowerCase().startsWith("win")) {
            return WINDOWS;
        } else if (os.toLowerCase().startsWith("mac")) {
            return MAC;
        }
        return LINUX;
    }

    /**
     * 文件大小转换
     */
    static String getSize(long size) {
        int KB = 1024;
        int GB = 1024 * 1024 * 1024;
        int MB = 1024 * 1024;
        DecimalFormat DF = new DecimalFormat("0.00");
        String resultSize;
        if (size / GB >= 1) {
            //如果当前Byte的值大于等于1GB
            resultSize = DF.format(size / (float) GB) + "GB   ";
        } else if (size / MB >= 1) {
            //如果当前Byte的值大于等于1MB
            resultSize = DF.format(size / (float) MB) + "MB   ";
        } else if (size / KB >= 1) {
            //如果当前Byte的值大于等于1KB
            resultSize = DF.format(size / (float) KB) + "KB   ";
        } else {
            resultSize = size + "B   ";
        }
        return resultSize;
    }

    static String getFileType(String type) {
        String IMAGE = "图片";
        String TXT = "文档";
        String MUSIC = "音乐";
        String VIDEO = "视频";
        String OTHER = "其他";
        String documents = "txt doc pdf ppt pps xlsx xls docx";
        String music = "mp3 wav wma mpa ram ra aac aif m4a";
        String video = "avi mpg mpe mpeg asf wmv mov qt rm mp4 flv m4v webm ogv ogg";
        String image = "bmp dib pcp dif wmf gif jpg tif eps psd cdr iff tga pcd mpt png jpeg";
        if (image.contains(type)) {
            return IMAGE;
        } else if (documents.contains(type)) {
            return TXT;
        } else if (music.contains(type)) {
            return MUSIC;
        } else if (video.contains(type)) {
            return VIDEO;
        } else {
            return OTHER;
        }
    }

}
