package com.flow.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flow.common.core.exception.BaseException;
import com.flow.common.core.model.PageResult;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.constant.FileConstant;
import com.flow.dao.FileDao;
import com.flow.entity.StorageFile;
import com.flow.enums.CategoryEnum;
import com.flow.mapstruct.StorageFileMapper;
import com.flow.model.StorageFileQuery;
import com.flow.service.FileService;
import com.flow.utils.FileUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.DigestUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;

@Service
public class FileServiceImpl extends BaseServiceImpl<FileDao, StorageFile> implements FileService {
    @Autowired
    private FileDao fileDao;
    @Autowired
    private StorageFileMapper storageFileMapper;

    @Override
    public PageResult<StorageFile> getList(StorageFileQuery storageFileQuery) {
        StorageFile storageFile = storageFileMapper.toEntity(storageFileQuery);
        Page<StorageFile> page = fileDao.lambdaQueryChain()
                .setEntity(storageFile)
                .page(
                        new Page<>(storageFileQuery.getPage(), storageFileQuery.getLimit())
                );
        return new PageResult<>(page.getTotal(), page.getRecords());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StorageFile upload(MultipartFile multipartFile) throws IOException {
        Long maxSize = FileConstant.MAX_SIZE;
        long fileSize = multipartFile.getSize();
        if (fileSize > maxSize) {
            throw new BaseException("文件超出规定大小:" + maxSize + "MB");
        }
        String filename = multipartFile.getOriginalFilename();
        String suffix = FileUtil.getSuffix(filename);
        File file = FileUtil.upload(multipartFile, FileConstant.getPath());
        String md5 = DigestUtils.md5DigestAsHex(Files.newInputStream(file.toPath()));
        ArrayList<String> doc = Lists.newArrayList("txt", "pdf", "doc", "docx", "ppt", "pptx", "xlsx", "xls");
        ArrayList<String> image = Lists.newArrayList("mpt", "pcd", "tga", "iff", "cdr", "psd", "eps", "tif", "wmf", "dif", "pcp", "dib", "jpg", "png", "gif", "jpeg", "bmp");
        ArrayList<String> video = Lists.newArrayList("flv", "dat", "m4v", "asx", "mov", "mp4", "avi", "rmvb", "rm", "asf", "divx", "mpg", "mpeg", "mpe", "wmv", "mkv", "vob");
        ArrayList<String> audio = Lists.newArrayList("mp3", "wma", "wav", "mid", "midi", "vqf", "amr", "aac", "flac", "ape", "ogg", "m4a", "ra", "3gp", "mp2");
        CategoryEnum category = CategoryEnum.OTHER;
        if (doc.contains(suffix.toLowerCase())) {
            category = CategoryEnum.DOC;
        } else if (image.contains(suffix.toLowerCase())) {
            category = CategoryEnum.IMAGE;
        } else if (video.contains(suffix.toLowerCase())) {
            category = CategoryEnum.VIDEO;
        } else if (audio.contains(suffix.toLowerCase())) {
            category = CategoryEnum.AUDIO;
        }
        String filePath = file.getPath();
        StorageFile storageFile = new StorageFile(
                filename,
                file.getName(),
                filePath,
                "/static/file/" + file.getName(),
                suffix,
                category,
                md5,
                fileSize,
                null,
                null

        );
        fileDao.insert(storageFile);
        return storageFile;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Set<Long> ids) {
        for (Long id : ids) {
            StorageFile storageFile = fileDao.selectById(id);
            if (Objects.nonNull(storageFile)) {
                List<StorageFile> list = fileDao.lambdaQueryChain()
                        .select(StorageFile::getId)
                        .eq(StorageFile::getMd5, storageFile.getMd5())
                        .list();
                if (list.size() <= 1) {
                    FileUtil.del(storageFile.getPath());
                }
                fileDao.deleteById(id);
            }
        }

    }
}
