package com.flow.service.impl;

import com.flow.service.NotificationStrategy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class NotificationStrategyFactory {

    @Autowired
    private List<NotificationStrategy> notificationStrategies;

    public <T> T getNotificationStrategy(Class<T> clazz) {
        for (NotificationStrategy notificationStrategy : notificationStrategies) {
            if (clazz.isInstance(notificationStrategy)) {
                return clazz.cast(notificationStrategy);
            }
        }
        return null;
    }

    public List<NotificationStrategy> getNotificationStrategy(Set<String> types) {
        return notificationStrategies.stream()
                .filter(strategy -> types.contains(strategy.getType()))
                .collect(Collectors.toList());
    }
}
