package com.flow.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.common.core.model.OauthUserDetails;
import com.flow.dao.AuthDao;
import com.flow.entity.Role;
import com.flow.entity.User;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Primary
@Service
public class UserDetailsServiceImpl implements UserDetailsService {
    @Autowired
    private AuthDao authDao;

    @Override
    public UserDetails loadUserByUsername(String userId) throws UsernameNotFoundException {
        User user = Optional.ofNullable(authDao.queryUserByUserId(userId))
                .orElseThrow(() -> new UsernameNotFoundException("账户或密码错误"));
        Set<Role> roles = "admin".equals(userId) ?
                authDao.getRoleAll() :
                authDao.queryRoleByUserId(userId);
        Set<GrantedAuthority> authorities = roles.stream()
                .map(role -> "ROLE_" + role.getName())
                .collect(Collectors.toSet())
                .stream().map(SimpleGrantedAuthority::new).collect(Collectors.toSet());
        Set<Long> roleIds = roles.stream().map(Role::getId).collect(Collectors.toSet());
        Set<String> permissions;
        if ("admin".equals(userId)) {
            permissions = authDao.getPermissionAll();
        } else {
            permissions = CollectionUtils.isNotEmpty(roleIds) ? authDao.queryPermissionsByRoleIds(roleIds) : new HashSet<>();
        }
        Set<SimpleGrantedAuthority> authoritySet = permissions.stream()
                .map(SimpleGrantedAuthority::new)
                .collect(Collectors.toSet());
        authorities.addAll(authoritySet);
        return new OauthUserDetails(
                user.getUsername(),
                user.getPassword(),
                user.getName(),
                user.getLeader(),
                user.getRoleIds(),
                user.getDeptId(),
                user.getPhone(),
                user.getEmail(),
                user.getEnabled(),
                authorities
        );
    }
}
