package com.flow.manger;

import com.flow.common.core.exception.BaseException;
import com.flow.entity.ReportDataSource;
import lombok.NoArgsConstructor;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.ReentrantLock;

@NoArgsConstructor
public class DataSourceManager {
    private static final Map<String, ReportDataSource> poolMap = new ConcurrentHashMap<>(100);
    //连接管理
    private static final ThreadLocal<Connection> CONNECTION_THREAD_LOCAL = new ThreadLocal<>();
    //线程锁
    private static final ReentrantLock poolLock = new ReentrantLock();

    public static Connection getConnection(final ReportDataSource reportDataSource) throws SQLException {
        DataSource dataSource = reportDataSource.buildDataSource();
        Connection connection = dataSource.getConnection();
        CONNECTION_THREAD_LOCAL.set(connection);
        String key = reportDataSource.getUrl();
        if (!poolMap.containsKey(key)) {
            poolLock.lock();
            try {
                if (!poolMap.containsKey(key)) {
                    poolMap.put(key, reportDataSource);
                }
            } finally {
                poolLock.unlock();
            }
        }
        return connection;
    }

    /**
     * 销毁连接(获取连接使用完成后需要调用该方法)
     */
    public static void release() {
        try {
            Connection conn = CONNECTION_THREAD_LOCAL.get();
            if (conn != null) {
                conn.close();
            }
        } catch (final SQLException ex) {
            throw new BaseException("数据库连接释放异常");
        } finally {
            CONNECTION_THREAD_LOCAL.remove();
        }
    }
}
