package com.flow.model.spreadsheet;

import com.fasterxml.jackson.annotation.JsonAnyGetter;
import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Data;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFFont;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Data
@JsonInclude(JsonInclude.Include.NON_NULL)
public class SheetFont {
    private Boolean bold;
    private Boolean italic;
    private Integer size;

    protected Map<String, Object> other = new HashMap<>();

    @JsonAnyGetter
    public Map<String, Object> getOther() {
        return other;
    }

    @JsonAnySetter
    public void setOther(String name, Object value) {
        this.other.put(name, value);
    }

    public XSSFFont toFont(SXSSFWorkbook workbook) {
        XSSFFont font = (XSSFFont) workbook.createFont();
        if (Objects.nonNull(this.bold)) {
            font.setBold(this.bold);
        }
        if (Objects.nonNull(this.size)) {
            font.setFontHeightInPoints(size.shortValue());
        }
        if (Objects.nonNull(this.italic)) {
            font.setItalic(this.italic);
        }
        return font;
    }
}
