package com.flow.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flow.common.core.model.PageResult;
import com.flow.common.mybatis.constant.SqlConstant;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.dao.DictDetailDao;
import com.flow.entity.DictDetail;
import com.flow.mapstruct.DictDetailMapper;
import com.flow.model.DictDetailQuery;
import com.flow.service.DictDetailService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.Set;

@Service
public class DictDetailServiceImpl extends BaseServiceImpl<DictDetailDao, DictDetail> implements DictDetailService {
    @Autowired
    private DictDetailDao dictDetailDao;
    @Autowired
    private DictDetailMapper dictDetailMapper;

    @Override
    public PageResult<DictDetail> getList(DictDetailQuery dictDetailQuery) {
        DictDetail dictDetail = dictDetailMapper.toEntity(dictDetailQuery);
        Page<DictDetail> page = dictDetailDao.lambdaQueryChain()
                .setEntity(dictDetail)
                .page(
                        new Page<>(dictDetailQuery.getPage(), dictDetailQuery.getLimit())
                );
        return new PageResult<>(page.getTotal(), page.getRecords());
    }

    @Override
    public List<DictDetail> getByDictCode(String dictCode) {
        return dictDetailDao.lambdaQueryChain()
                .eq(DictDetail::getDictCode, dictCode)
                .list();
    }

    @Override
    public DictDetail create(DictDetail dictDetail) {
        Optional<DictDetail> optional = dictDetailDao.lambdaQueryChain()
                .eq(DictDetail::getDictCode, dictDetail.getDictCode())
                .eq(DictDetail::getValue, dictDetail.getValue())
                .last(SqlConstant.LIMIT1)
                .oneOpt();
        if (optional.isPresent()) {
            throw new RuntimeException("字典值已存在");
        }
        dictDetailDao.insert(dictDetail);
        return dictDetail;
    }

    @Override
    public DictDetail update(DictDetail dictDetail) {
        Optional<DictDetail> optional = dictDetailDao.lambdaQueryChain()
                .eq(DictDetail::getDictCode, dictDetail.getDictCode())
                .eq(DictDetail::getValue, dictDetail.getValue())
                .last(SqlConstant.LIMIT1)
                .oneOpt();
        if (optional.isPresent()) {
            throw new RuntimeException("字典值已存在");
        }
        dictDetailDao.updateById(dictDetail);
        return dictDetail;
    }

    @Override
    public void delete(Set<Long> ids) {
        dictDetailDao.deleteBatchIds(ids);
    }
}
