package com.flow.service.impl;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flow.common.core.exception.BaseException;
import com.flow.common.core.model.PageResult;
import com.flow.common.mybatis.constant.SqlConstant;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.dao.DictDao;
import com.flow.entity.Dict;
import com.flow.mapstruct.DictMapper;
import com.flow.model.DictQuery;
import com.flow.service.DictService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.Set;

@Service
public class DictServiceImpl extends BaseServiceImpl<DictDao, Dict> implements DictService {
    @Autowired
    private DictDao dictDao;
    @Autowired
    private DictMapper dictMapper;

    @Override
    public PageResult<Dict> getList(DictQuery dictQuery) {
        Dict dict = dictMapper.toEntity(dictQuery);
        Page<Dict> page = dictDao.lambdaQueryChain()
                .setEntity(dict)
                .page(
                        new Page<>(dictQuery.getPage(), dictQuery.getLimit())
                );
        return new PageResult<>(page.getTotal(), page.getRecords());
    }

    @Override
    public Dict create(Dict dict) {
        Optional<Dict> optional = dictDao.lambdaQueryChain()
                .eq(Dict::getCode, dict.getCode())
                .last(SqlConstant.LIMIT1)
                .oneOpt();
        if (optional.isPresent()) {
            throw new BaseException("字典编码已存在");
        }
        dictDao.insert(dict);
        return dict;
    }

    @Override
    public Dict update(Dict dict) {
        dictDao.updateById(dict);
        List<Dict> list = dictDao.lambdaQueryChain()
                .select(Dict::getId)
                .eq(Dict::getCode, dict.getCode())
                .last(SqlConstant.LIMIT2)
                .list();
        if (list.size() > 1) {
            throw new BaseException("字典编码已存在");
        }
        return dict;
    }

    @Override
    public void delete(Set<Long> ids) {
        dictDao.deleteBatchIds(ids);
    }
}
