package com.flow.controller;

import com.flow.common.core.model.PageResult;
import com.flow.common.core.model.Result;
import com.flow.entity.DictDetail;
import com.flow.model.DictDetailQuery;
import com.flow.service.DictDetailService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;


@RestController
@RequestMapping("/dictDetail")
public class DictDetailController {
    @Autowired
    private DictDetailService dictDetailService;

    @GetMapping
    public Result<PageResult<DictDetail>> getList(DictDetailQuery dictDetailQuery) {
        return Result.success(dictDetailService.getList(dictDetailQuery));
    }

    @GetMapping("/dict/{dictCode}")
    public Result<List<DictDetail>> getListByDictCode(@PathVariable String dictCode) {
        return Result.success(dictDetailService.getByDictCode(dictCode));
    }

    @PreAuthorize("hasAnyAuthority('dict:add')")
    @PostMapping
    public Result<DictDetail> create(@RequestBody DictDetail dictDetail) {
        return Result.success(dictDetailService.create(dictDetail));
    }

    @PreAuthorize("hasAnyAuthority('dict:edit')")
    @PutMapping
    public Result<DictDetail> update(@RequestBody DictDetail dictDetail) {
        return Result.success(dictDetailService.update(dictDetail));
    }

    @PreAuthorize("hasAnyAuthority('dict:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        dictDetailService.delete(ids);
        return Result.success();
    }
}
