package com.flow.controller;

import com.flow.common.core.model.PageResult;
import com.flow.common.core.model.Result;
import com.flow.entity.Role;
import com.flow.entity.StorageFile;
import com.flow.entity.User;
import com.flow.model.UserQuery;
import com.flow.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Set;

@RestController
@RequestMapping("/user")
public class UserController {
    @Autowired
    private UserService userService;

    @GetMapping("/list")
    public Result<List<User>> list() {
        List<User> list = userService.lambdaQuery()
                .ne(User::getUsername, "admin")
                .list();
        return Result.success(list);
    }

    @GetMapping("/search")
    public Result<List<User>> search(UserQuery userQuery) {
        return Result.success(userService.search(userQuery));
    }

    @GetMapping("/role/{userId}")
    public Result<List<Role>> getRoles(@PathVariable String userId) {
        return Result.success(userService.getRoles(userId));
    }

    @PostMapping("/role/{userId}")
    public Result<?> authRoles(@PathVariable String userId, @RequestBody Set<String> roleIds) {
        userService.authRoles(userId, roleIds);
        return Result.success();
    }

    @GetMapping
    public Result<PageResult<User>> getPage(UserQuery userQuery) {
        return Result.success(userService.getList(userQuery));
    }

    @PostMapping("/options")
    public Result<List<User>> getOptions(@RequestBody Set<String> userIds) {
        List<User> options = userService.getOptions(userIds);
        return Result.success(options);
    }

    @GetMapping("/{username}")
    public Result<User> getByUsername(@PathVariable String username) {
        return Result.success(userService.getByUsername(username));
    }

    @PreAuthorize("hasAnyAuthority('user:add')")
    @PostMapping
    public Result<User> create(@RequestBody User user) {
        return Result.success(userService.create(user));
    }

    @PreAuthorize("hasAnyAuthority('user:edit')")
    @PutMapping
    public Result<User> update(@RequestBody User user) {
        return Result.success(userService.update(user));
    }

    @PreAuthorize("hasAnyAuthority('user:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        userService.delete(ids);
        return Result.success();
    }

    @PostMapping("/avatar")
    public Result<StorageFile> updateAvatar(@RequestParam("avatar") MultipartFile multipartFile) throws IOException {
        StorageFile storageFile = userService.updateAvatar(multipartFile);
        return Result.success(storageFile);
    }

    @PreAuthorize("hasAnyAuthority('user:resetPwd')")
    @PutMapping("/reset/{username}")
    public Result<String> resetPassword(@PathVariable String username) {
        userService.resetPassword(username);
        return Result.success("重置密码成功，新密码为“123456”");
    }

}
