package com.flow.service;


import com.flow.common.core.model.PageResult;
import com.flow.common.mybatis.service.BaseService;
import com.flow.entity.FlowTask;
import com.flow.entity.node.Node;
import com.flow.enums.ProcessStatus;
import com.flow.model.*;
import org.flowable.task.api.Task;
import org.flowable.task.api.TaskInfo;

import java.util.List;
import java.util.Map;
import java.util.Set;

public interface FlowTaskService extends BaseService<FlowTask> {
    /**
     * 待办
     * @param flowTaskQuery
     * @return
     */
    PageResult<FlowTask> todo(FlowTaskQuery flowTaskQuery);

    /**
     * 已办
     * @param flowTaskQuery
     * @return
     */
    PageResult<FlowTask> done(FlowTaskQuery flowTaskQuery);

    /**
     * 任务表单
     * @param taskId
     * @return
     */
    FormInfo getForm(String taskId);

    /**
     * 保存表单
     * @param taskForm
     */
    void saveForm(TaskForm taskForm);

    /**
     * 评论
     * @param taskForm
     * @return
     */
    Task comment(TaskForm taskForm);

    /**
     * 获取评论
     * @param taskId
     * @return
     */
    List<FlowComment> getComments(String taskId);

    /**
     * 获取任务
     * @param taskId
     * @return
     */
    TaskInfo getTask(String taskId);

    /**
     * 获取流程变量
     * @param taskId
     * @return
     */
    Map<String, Object> getVariables(String taskId);

    /**
     * 回退/驳回
     * @param taskForm
     */
    void reject(RejectTaskForm taskForm);

    /**
     * 拒绝
     * @param taskForm
     */
    void refuse(TaskForm taskForm);

    /**
     * 自动拒绝
     * @param taskId
     */
    void autoRefuse(String taskId);

    /**
     * 自动拒绝
     * @param taskId
     * @param content
     */
    void autoRefuse(String taskId, String content);

    /**
     * 转办
     * @param taskForm
     */
    void transfer(TransferTaskForm taskForm);

    /**
     * 加签
     * @param taskForm
     */
    void addSign(AddSignTaskForm taskForm);

    /**
     * 减签
     * @param taskForm
     */
    void removeSign(AddSignTaskForm taskForm);

    /**
     * 委派
     * @param taskForm
     */
    void delegate(TransferTaskForm taskForm);

    /**
     * 更新状态
     * @param id
     * @param status
     */
    void updateStatus(String id, ProcessStatus status);

    /**
     * 更新状态
     * @param id
     * @param status
     * @param variables
     */
    void updateStatus(String id, ProcessStatus status, Map<String, Object> variables);

    /**
     * 完成任务
     * @param taskForm
     */
    void complete(TaskForm taskForm);

    /**
     * 自动完成任务
     * @param taskId
     */
    void autoComplete(String taskId);

    /**
     * 自动完成任务
     * @param taskId
     * @param content
     */
    void autoComplete(String taskId, String content);

    /**
     * 获取父节点
     * @param taskId
     * @return
     */
    List<Node> getParentNodes(String taskId);

    /**
     * 获取多实例任务未完成用户
     * @param taskId
     * @return
     */
    Set<String> getMultiInstanceTaskUnUsers(String taskId);

}
