package com.flow.delegate;

import com.flow.common.core.util.StrUtil;
import com.flow.entity.FlowDefine;
import com.flow.entity.Notify;
import com.flow.entity.User;
import com.flow.entity.node.NotifyNode;
import com.flow.enums.NotifyEnum;
import com.flow.enums.NotifyTypeEnum;
import com.flow.service.FlowDefineService;
import com.flow.service.NotifyService;
import com.flow.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.JavaDelegate;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@Slf4j
@Component
public class NotifyDelegate implements JavaDelegate {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private JavaMailSender mailSender;
    @Value("${spring.mail.username}")
    private String from;
    @Autowired
    private NotifyService notifyService;
    @Autowired
    private UserService userService;


    @Override
    public void execute(DelegateExecution execution) {
        ExecutionEntityImpl entity = (ExecutionEntityImpl) execution;
        String activityId = execution.getCurrentActivityId();
        Map<String, Object> variables = execution.getVariables();
        FlowDefine define = flowDefineService.getDefine(entity.getProcessDefinitionId());
        NotifyNode notifyNode = define.getNode(execution.getCurrentActivityId(), NotifyNode.class);
        List<NotifyTypeEnum> typeEnums = notifyNode.getTypes();
        // 通知
        ArrayList<String> assignees = execution.getVariable(String.format("%sCollection", activityId), ArrayList.class);
        List<User> userList = userService.lambdaQuery()
                .in(User::getUsername, assignees)
                .list();
        for (NotifyTypeEnum typeEnum : typeEnums) {
            switch (typeEnum) {
                case SITE:
                    // 站内通知
                    for (User assignee : userList) {
                        Notify notify = new Notify();
                        notify.setSender("admin");
                        notify.setType(NotifyEnum.SYSTEM);
                        notify.setReceivingTime(LocalDateTime.now());
                        notify.setReceiver(assignee.getUsername());
                        notify.setSubject(StrUtil.format(notifyNode.getSubject(), variables));
                        notify.setContent(StrUtil.format(notifyNode.getContent(), variables));
                        notifyService.notify(notify);
                    }
                    break;
                case EMAIL:
                    // 邮件通知
                    for (User assignee : userList) {
                        SimpleMailMessage message = new SimpleMailMessage();
                        message.setFrom(this.from);
                        message.setTo(assignee.getEmail());
                        message.setSubject(StrUtil.format(notifyNode.getSubject(), variables));
                        message.setText(StrUtil.format(notifyNode.getContent(), variables));
                        mailSender.send(message);
                    }
                    break;
                default:
                    break;
            }
        }
    }
}
