package com.flow.listener;

import com.flow.entity.FlowDefine;
import com.flow.entity.Notify;
import com.flow.entity.node.ApprovalNode;
import com.flow.entity.node.TimerBoundary;
import com.flow.enums.ApprovalTimeoutEnum;
import com.flow.enums.NotifyEnum;
import com.flow.enums.TimeUnitEnum;
import com.flow.service.FlowDefineService;
import com.flow.service.FlowTaskService;
import com.flow.service.NotifyService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.TaskService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.ExecutionListener;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * 审批超时处理
 */
@Slf4j
@Component
public class ApprovalTimeoutListener implements ExecutionListener {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private FlowTaskService flowTaskService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private NotifyService notifyService;

    @Override
    public void notify(DelegateExecution delegateExecution) {
        String currentActivityId = delegateExecution.getCurrentActivityId();
        FlowDefine define = flowDefineService.getDefine(delegateExecution.getProcessDefinitionId());
        String nodeId = currentActivityId.replaceAll("_boundary", "");
        ApprovalNode approvalNode = define.getNode(nodeId, ApprovalNode.class);
        TimerBoundary timeout = approvalNode.getTimeout();
        Boolean enabled = timeout.getEnabled();
        if (Objects.nonNull(enabled) && enabled) {
            ApprovalTimeoutEnum timeoutAction = timeout.getAction();
            Integer duration = timeout.getDuration();
            TimeUnitEnum unit = timeout.getUnit();
            List<Task> taskList = taskService.createTaskQuery()
                    .processInstanceId(delegateExecution.getProcessInstanceId())
                    .taskDefinitionKey(nodeId)
                    .list();
            switch (timeoutAction) {
                case AUTO_PASS:
                    // 自动通过
                    for (Task task : taskList) {
                        flowTaskService.autoComplete(task.getId());
                    }
                    break;
                case AUTO_REFUSE:
                    // 自动拒绝
                    for (Task task : taskList) {
                        flowTaskService.autoRefuse(task.getId());
                    }
                    break;
                case REMIND:
                    // 提醒
                    for (Task task : taskList) {
                        Notify notify = new Notify();
                        notify.setSender("admin");
                        notify.setType(NotifyEnum.SYSTEM);
                        notify.setReceivingTime(LocalDateTime.now());
                        notify.setReceiver(task.getAssignee());
                        notify.setUrl("/flow/todo");
                        notify.setSubject("审批提醒");
                        notify.setContent(String.format("您的审批任务【%s】已超%s%s未处理，请及时处理。", task.getName(), duration, unit.getDescription()));
                        notifyService.notify(notify);
                    }
                    break;
                default:
                    break;
            }
        }
    }
}
