package com.flow.service.impl;

import com.flow.entity.Activity;
import com.flow.entity.FlowDefine;
import com.flow.entity.FlowTask;
import com.flow.entity.node.ApprovalNode;
import com.flow.entity.node.JumpNode;
import com.flow.entity.node.Node;
import com.flow.enums.ApprovalMultiEnum;
import com.flow.service.FlowActivityService;
import com.flow.service.FlowDefineService;
import com.flow.service.FlowTaskService;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.history.HistoricActivityInstance;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.engine.runtime.ActivityInstance;
import org.flowable.engine.runtime.ProcessInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class FlowActivityServiceImpl implements FlowActivityService {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private FlowTaskService flowTaskService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private HistoryService historyService;

    @Override
    public List<Activity> records(String instanceId) {
        ProcessInstance processInstance = runtimeService.createProcessInstanceQuery().processInstanceId(instanceId).singleResult();
        if (Objects.isNull(processInstance)) {
            return historyRecords(instanceId);
        }
        FlowDefine flowDefine = flowDefineService.getDefine(processInstance.getProcessDefinitionId());
        List<FlowTask> flowTasks = flowTaskService.lambdaQuery()
                .select(FlowTask::getId, FlowTask::getStatus)
                .eq(FlowTask::getInstanceId, instanceId)
                .list();
        List<ActivityInstance> list = runtimeService.createActivityInstanceQuery()
                .processInstanceId(instanceId)
                .list();
        return list.stream()
                .map(e -> {
                    Activity activity = new Activity(e);
                    activity.setMulti(ApprovalMultiEnum.SEQUENTIAL);
                    if ("userTask".equals(activity.getActivityType())) {
                        Optional<FlowTask> optional = flowTasks.stream().filter(t -> t.getId().equals(e.getTaskId())).findFirst();
                        optional.ifPresent(flowTask -> activity.setStatus(flowTask.getStatus()));
                        String assignee = activity.getAssignee();
                        if (Objects.isNull(assignee)) {
                            activity.setAssignee(processInstance.getStartUserId());
                        }
                        ApprovalNode approvalNode = flowDefine.findFlowNodeOfType(ApprovalNode.class, e.getActivityId());
                        if (Objects.nonNull(approvalNode)) {
                            ApprovalMultiEnum multi = approvalNode.getMulti();
                            activity.setMulti(multi);
                        }
                    } else if ("copyServiceTask".equals(activity.getActivityType())) {
                        activity.setMulti(ApprovalMultiEnum.JOINT);
                    } else if ("startEvent".equals(activity.getActivityType())) {
                        activity.setAssignee(processInstance.getStartUserId());
                    } else if ("jumpServiceTask".equals(activity.getActivityType())) {
                        JumpNode jumpNode = flowDefine.findFlowNodeOfType(JumpNode.class, e.getActivityId());
                        Node target = flowDefine.findFlowNodeOfType(Node.class, jumpNode.getTargetNode());
                        activity.setAssignee("admin");
                        activity.setDescription(String.format("<span>跳转至 <span class='color-primary'>%s</span></span>", target.getName()));
                    }
                    return activity;
                })
                .sorted(Comparator.comparing(Activity::getId))
                .collect(Collectors.toList());
    }

    @Override
    public List<Activity> historyRecords(String instanceId) {
        HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(instanceId)
                .singleResult();
        FlowDefine flowDefine = flowDefineService.getDefine(historicProcessInstance.getProcessDefinitionId());
        List<FlowTask> flowTasks = flowTaskService.lambdaQuery()
                .select(FlowTask::getId, FlowTask::getStatus)
                .eq(FlowTask::getInstanceId, instanceId)
                .list();
        List<HistoricActivityInstance> list = historyService.createHistoricActivityInstanceQuery()
                .processInstanceId(instanceId)
                .list();
        return list.stream()
                .map(e -> {
                    Activity activity = new Activity(e);
                    activity.setMulti(ApprovalMultiEnum.SEQUENTIAL);
                    if ("userTask".equals(activity.getActivityType())) {
                        Optional<FlowTask> optional = flowTasks.stream().filter(t -> t.getId().equals(e.getTaskId())).findFirst();
                        optional.ifPresent(flowTask -> activity.setStatus(flowTask.getStatus()));
                        String assignee = activity.getAssignee();
                        if (Objects.isNull(assignee)) {
                            activity.setAssignee(historicProcessInstance.getStartUserId());
                        }
                        ApprovalNode approvalNode = flowDefine.findFlowNodeOfType(ApprovalNode.class, e.getActivityId());
                        if (Objects.nonNull(approvalNode)) {
                            ApprovalMultiEnum multi = approvalNode.getMulti();
                            activity.setMulti(multi);
                        }
                    } else if ("copyServiceTask".equals(activity.getActivityType())) {
                        activity.setMulti(ApprovalMultiEnum.JOINT);
                    } else if ("startEvent".equals(activity.getActivityType())) {
                        activity.setAssignee(historicProcessInstance.getStartUserId());
                    } else if ("jumpServiceTask".equals(activity.getActivityType())) {
                        JumpNode jumpNode = flowDefine.findFlowNodeOfType(JumpNode.class, e.getActivityId());
                        Node target = flowDefine.findFlowNodeOfType(Node.class, jumpNode.getTargetNode());
                        activity.setAssignee("admin");
                        activity.setDescription(String.format("<span>跳转至 <span class='color-primary'>%s</span></span>", target.getName()));
                    }
                    return activity;
                })
                .sorted(Comparator.comparing(Activity::getId))
                .collect(Collectors.toList());
    }
}
