package com.flow.controller;

import com.flow.common.core.model.Result;
import com.flow.entity.FlowGroup;
import com.flow.service.FlowGroupService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;

@RestController
@RequestMapping("/group")
public class FlowGroupController {
    @Autowired
    private FlowGroupService flowGroupService;

    @GetMapping
    public Result<List<FlowGroup>> list(FlowGroup flowGroup) {
        return Result.success(flowGroupService.list(flowGroup));
    }

    @PreAuthorize("hasAnyAuthority('flow:group:add')")
    @PostMapping
    public Result<FlowGroup> create(@RequestBody FlowGroup flowGroup) {
        return Result.success(flowGroupService.create(flowGroup));
    }

    @PreAuthorize("hasAnyAuthority('flow:group:edit')")
    @PutMapping
    public Result<FlowGroup> update(@RequestBody FlowGroup flowGroup) {
        return Result.success(flowGroupService.update(flowGroup));
    }

    @PreAuthorize("hasAnyAuthority('flow:group:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        flowGroupService.delete(ids);
        return Result.success();
    }

    @PreAuthorize("hasAnyAuthority('flow:group:edit')")
    @PutMapping("/sort/{newId}/{oldId}")
    public Result<?> sort(@PathVariable String newId, @PathVariable String oldId) {
        flowGroupService.sort(newId,oldId);
        return Result.success();
    }

}