package com.flow.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.flow.common.core.model.PageResult;
import com.flow.common.core.model.Result;
import com.flow.common.core.util.SecurityContextUtil;
import com.flow.entity.FlowInstance;
import com.flow.model.FlowInstanceQuery;
import com.flow.model.FormInfo;
import com.flow.model.spreadsheet.SpreadSheet;
import com.flow.service.FlowInstanceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/instance")
public class FlowInstanceController {
    @Autowired
    private FlowInstanceService flowInstanceService;

    @GetMapping
    public Result<PageResult<FlowInstance>> list(FlowInstanceQuery flowInstanceQuery) {
        flowInstanceQuery.setStartUserId(SecurityContextUtil.getUserId());
        PageResult<FlowInstance> pageResult = flowInstanceService.list(flowInstanceQuery);
        return Result.success(pageResult);
    }

    @GetMapping("/copy")
    public Result<PageResult<FlowInstance>> copy(FlowInstanceQuery flowInstanceQuery) {
        PageResult<FlowInstance> pageResult = flowInstanceService.copy(flowInstanceQuery);
        return Result.success(pageResult);
    }

    @GetMapping("/{instanceId}")
    public Result<FlowInstance> getInstance(@PathVariable String instanceId) {
        return Result.success(flowInstanceService.getInstance(instanceId));
    }

    @GetMapping("/variables/{instanceId}")
    public Result<Map<String, Object>> getVariables(@PathVariable String instanceId) {
        return Result.success(flowInstanceService.getVariables(instanceId));
    }

    @GetMapping("/form/{instanceId}")
    public Result<FormInfo> getForm(@PathVariable String instanceId) {
        return Result.success(flowInstanceService.getForm(instanceId));
    }

    @PutMapping("/cancel")
    public Result<?> cancel(@RequestBody List<String> instanceIds) {
        flowInstanceService.cancel(instanceIds);
        return Result.success();
    }

    @PutMapping("/suspend/{instanceId}")
    public Result<?> suspend(@PathVariable String instanceId) {
        flowInstanceService.suspend(instanceId);
        return Result.success();
    }

    @GetMapping("/print/{instanceId}")
    public Result<SpreadSheet> getPrintData(@PathVariable String instanceId) throws JsonProcessingException {
        return Result.successData(flowInstanceService.getPrintData(instanceId));
    }

    @GetMapping("/print/exist/{instanceId}")
    public Result<Boolean> getPrintExist(@PathVariable String instanceId) throws JsonProcessingException {
        return Result.successData(flowInstanceService.getPrintExist(instanceId));
    }
}
