package com.flow.entity;

import com.flow.enums.ApprovalMultiEnum;
import com.flow.enums.ProcessStatus;
import lombok.Data;
import org.flowable.engine.history.HistoricActivityInstance;
import org.flowable.engine.runtime.ActivityInstance;

import java.util.Date;
import java.util.Objects;

@Data
public class Activity {
    private String id;
    private String defineId;
    private ApprovalMultiEnum multi;
    private String taskId;
    private String instanceId;
    private String executionId;
    private String activityId;
    private String activityName;
    private String activityType;
    private String assignee;
    private Date startTime;
    private Date endTime;
    private Long duration;
    private String description;
    private ProcessStatus status;

    public Activity(ActivityInstance activityInstance) {
        this.id = activityInstance.getId();
        this.activityId = activityInstance.getActivityId();
        this.activityName = activityInstance.getActivityName();
        this.activityType = activityInstance.getActivityType();
        this.assignee = activityInstance.getAssignee();
        this.startTime = activityInstance.getStartTime();
        this.endTime = activityInstance.getEndTime();
        this.duration = activityInstance.getDurationInMillis();
        this.executionId = activityInstance.getExecutionId();
        this.instanceId = activityInstance.getProcessInstanceId();
        this.taskId = activityInstance.getTaskId();
        this.defineId = activityInstance.getProcessDefinitionId();
        if (Objects.isNull(this.endTime)) {
            this.status = ProcessStatus.RUNNING;
        } else {
            this.status = ProcessStatus.COMPLETED;
        }
    }

    public Activity(HistoricActivityInstance historicActivityInstance) {
        this.id = historicActivityInstance.getId();
        this.activityId = historicActivityInstance.getActivityId();
        this.assignee = historicActivityInstance.getAssignee();
        this.duration = historicActivityInstance.getDurationInMillis();
        this.startTime = historicActivityInstance.getStartTime();
        this.endTime = historicActivityInstance.getEndTime();
        this.defineId = historicActivityInstance.getProcessDefinitionId();
        this.instanceId = historicActivityInstance.getProcessInstanceId();
        this.executionId = historicActivityInstance.getExecutionId();
        this.taskId = historicActivityInstance.getTaskId();
        this.activityName = historicActivityInstance.getActivityName();
        this.activityType = historicActivityInstance.getActivityType();
        if (Objects.isNull(this.endTime)) {
            this.status = ProcessStatus.RUNNING;
        } else {
            this.status = ProcessStatus.COMPLETED;
        }
    }

}
