package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.common.core.exception.BaseException;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.Gateway;
import org.flowable.bpmn.model.SequenceFlow;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@EqualsAndHashCode(callSuper = true)
@Data
public abstract class BranchNode extends Node {
    private List<ConditionNode> children;

    public abstract List<FlowElement> convert();

    /**
     * 分支聚合
     *
     * @param gateway
     * @return
     */
    public List<FlowElement> merge(Gateway gateway) {
        ArrayList<FlowElement> elements = Lists.newArrayList(gateway);
        // 下一个节点的连线
        SequenceFlow sequenceFlow = new SequenceFlow();
        Node child = this.getChild();
        Optional<Node> optional = Optional.ofNullable(child);
        if (optional.isPresent()) {
            sequenceFlow.setSourceRef(gateway.getId());
            sequenceFlow.setTargetRef(child.getId());
            sequenceFlow.setId(String.format("%s-%s", gateway.getId(), child.getId()));
        } else {
            if (StringUtils.isNotBlank(this.getBranchId())) {
                sequenceFlow.setSourceRef(gateway.getId());
                sequenceFlow.setTargetRef(this.getBranchId());
                sequenceFlow.setId(String.format("%s-%s", gateway.getId(), this.getBranchId()));
            } else {
                throw new BaseException(String.format("节点 %s 的下一个节点不能为空", this.getId()));
            }
        }
        elements.add(sequenceFlow);
        return elements;
    }
}
