package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.ExclusiveGateway;
import org.flowable.bpmn.model.FlowElement;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@EqualsAndHashCode(callSuper = true)
@Data
public class ExclusiveNode extends BranchNode {

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 独占分支
        ExclusiveGateway exclusiveGateway = new ExclusiveGateway();
        exclusiveGateway.setId(this.getId());
        exclusiveGateway.setName(this.getName());
        exclusiveGateway.setExecutionListeners(this.buidEventListener());
        elements.add(exclusiveGateway);
        List<ConditionNode> children = this.getChildren();
        children.stream()
                .filter(ConditionNode::getDef)
                .findFirst()
                .ifPresent(conditionNode -> {
                    exclusiveGateway.setDefaultFlow(conditionNode.getId());
                });
        // 分支聚合
        ExclusiveGateway gateway = new ExclusiveGateway();
        gateway.setId(String.format("%send", this.getId()));
        gateway.setName("独占聚合");
        elements.addAll(this.merge(gateway));
        // 子节点
        if (CollectionUtils.isNotEmpty(children)) {
            for (Node next : children) {
                String branchId = gateway.getId();
                // String branchId = Optional.ofNullable(this.getChild()).map(Node::getId).orElse(this.getBranchId());
                next.setBranchId(branchId);
                elements.addAll(next.convert());
            }
        }
        // 下一个节点
        Node child = this.getChild();
        if (Objects.nonNull(child)) {
            child.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = child.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
