package com.flow.entity.node;

import com.flow.flowable.model.JumpServiceTask;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FieldExtension;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.SequenceFlow;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@EqualsAndHashCode(callSuper = true)
@Data
public class JumpNode extends Node {
    private String targetNode;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 服务节点
        JumpServiceTask jumpServiceTask = new JumpServiceTask();
        jumpServiceTask.setId(this.getId());
        jumpServiceTask.setType("jump");
        jumpServiceTask.setName(this.getName());
        jumpServiceTask.setExecutionListeners(this.buidEventListener());
        // jumpServiceTask.setImplementationType(ImplementationType.IMPLEMENTATION_TYPE_DELEGATEEXPRESSION);
        // jumpServiceTask.setImplementation("${jumpDelegate}");
        // 扩展属性
        FieldExtension fieldExtension = new FieldExtension();
        fieldExtension.setFieldName("targetNode");
        fieldExtension.setStringValue(this.targetNode);
        jumpServiceTask.getFieldExtensions().add(fieldExtension);
        elements.add(jumpServiceTask);
        // 下一个节点的连线
        Node child = this.getChild();
        SequenceFlow sequenceFlow = this.buildSequence(child);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(child)) {
            child.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = child.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
