package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeId;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.flow.common.core.exception.BaseException;
import com.flow.constant.NodeTypeConstant;
import com.google.common.collect.Lists;
import lombok.Data;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.FlowableListener;
import org.flowable.bpmn.model.SequenceFlow;

import java.io.Serializable;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Data
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, property = "type", defaultImpl = Node.class, visible = true)
@JsonSubTypes({
        @JsonSubTypes.Type(value = StartNode.class, name = NodeTypeConstant.START),
        @JsonSubTypes.Type(value = CcNode.class, name = NodeTypeConstant.CC),
        @JsonSubTypes.Type(value = ApprovalNode.class, name = NodeTypeConstant.APPROVAL),
        @JsonSubTypes.Type(value = ConditionNode.class, name = NodeTypeConstant.CONDITION),
        @JsonSubTypes.Type(value = ConditionNode.class, name = NodeTypeConstant.CONCURRENT),
        @JsonSubTypes.Type(value = ExclusiveNode.class, name = NodeTypeConstant.EXCLUSIVE),
        @JsonSubTypes.Type(value = ParallelNode.class, name = NodeTypeConstant.PARALLEL),
        @JsonSubTypes.Type(value = InclusiveNode.class, name = NodeTypeConstant.INCLUSIVE),
        @JsonSubTypes.Type(value = TimerNode.class, name = NodeTypeConstant.TIMER),
        @JsonSubTypes.Type(value = NotifyNode.class, name = NodeTypeConstant.NOTIFY),
        @JsonSubTypes.Type(value = JumpNode.class, name = NodeTypeConstant.JUMP),
        @JsonSubTypes.Type(value = EndNode.class, name = NodeTypeConstant.END)
})
public abstract class Node implements Serializable {
    private static final long serialVersionUID = 132324315232123L;
    private String id;
    private String pid;
    private String name;
    @JsonTypeId
    private String type;
    private List<NodeListener> executionListeners;
    private Node child;
    @JsonIgnore
    private String branchId;

    public abstract List<FlowElement> convert();

    public List<FlowableListener> buidEventListener() {
        if (CollectionUtils.isNotEmpty(this.executionListeners)) {
            return this.executionListeners.stream().filter(l -> StringUtils.isNotBlank(l.getImplementation())).map(listener -> {
                FlowableListener executionListener = new FlowableListener();
                executionListener.setEvent(listener.getEvent());
                executionListener.setImplementationType(listener.getImplementationType());
                executionListener.setImplementation(listener.getImplementation());
                return executionListener;
            }).collect(Collectors.toList());
        }
        return Lists.newArrayList();
    }

    public SequenceFlow buildSequence(Node next) {
        String sourceRef;
        String targetRef;
        if (Objects.nonNull(next)) {
            sourceRef = next.getPid();
            targetRef = next.getId();
        } else {
            if (StringUtils.isNotBlank(this.branchId)) {
                sourceRef = this.id;
                targetRef = this.branchId;
            } else {
                throw new BaseException(String.format("节点 %s 的下一个节点不能为空", this.id));
            }
        }
        SequenceFlow sequenceFlow = new SequenceFlow();
        sequenceFlow.setId(String.format("%s-%s", sourceRef, targetRef));
        sequenceFlow.setSourceRef(sourceRef);
        sequenceFlow.setTargetRef(targetRef);
        return sequenceFlow;
    }
}
