package com.flow.entity.node;

import com.flow.enums.ApprovalTimeoutEnum;
import com.flow.enums.TimeUnitEnum;
import com.google.common.collect.Lists;
import lombok.Data;
import org.flowable.bpmn.model.*;

@Data
public class TimerBoundary {
    private Boolean enabled;
    private Integer duration;
    private TimeUnitEnum unit;
    private ApprovalTimeoutEnum action;
    private Boolean loop;


    public BoundaryEvent convert(Activity activity) {
        BoundaryEvent boundaryEvent = new BoundaryEvent();
        boundaryEvent.setId(String.format("%s_boundary", activity.getId()));
        boundaryEvent.setName("审批超时");
        boundaryEvent.setCancelActivity(false);
        boundaryEvent.setAttachedToRef(activity);
        // 计时器
        TimerEventDefinition timerEventDefinition = new TimerEventDefinition();
        if (action == ApprovalTimeoutEnum.REMIND && loop) {
            timerEventDefinition.setTimeCycle("R/" + String.format(this.getUnit().getUnit(), this.getDuration()));
        } else {
            timerEventDefinition.setTimeDuration(String.format(this.getUnit().getUnit(), this.getDuration()));
        }
        boundaryEvent.setEventDefinitions(Lists.newArrayList(timerEventDefinition));
        // 边界事件定时器
        FlowableListener timerListener = new FlowableListener();
        timerListener.setEvent("end");
        timerListener.setImplementationType(ImplementationType.IMPLEMENTATION_TYPE_DELEGATEEXPRESSION);
        timerListener.setImplementation("${approvalTimeoutListener}");
        boundaryEvent.setExecutionListeners(Lists.newArrayList(timerListener));
        return boundaryEvent;
    }
}
