package com.flow.model;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.flowable.engine.task.Attachment;
import org.flowable.engine.task.Comment;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Data
@NoArgsConstructor
public class FlowComment {
    // 流程实例id
    private String instanceId;
    // 任务id
    private String taskId;
    // 评论内容
    private String content;
    // 评论人
    private String userId;
    // 评论时间
    private Date time;
    // 评论附件
    private List<FlowAttachment> attachments;

    public FlowComment(Comment comment, List<Attachment> attachments) {
        this.setContent(comment.getFullMessage());
        this.setUserId(comment.getUserId());
        this.setTime(comment.getTime());
        this.setTaskId(comment.getTaskId());
        this.setInstanceId(comment.getProcessInstanceId());
        if (!CollectionUtils.isEmpty(attachments)) {
            List<FlowAttachment> flowAttachmentList = attachments.stream().map(attachment -> {
                FlowAttachment flowAttachment = new FlowAttachment();
                flowAttachment.setType(attachment.getType());
                flowAttachment.setName(attachment.getName());
                flowAttachment.setUrl(attachment.getUrl());
                return flowAttachment;
            }).collect(Collectors.toList());
            this.setAttachments(flowAttachmentList);
        }
    }
}
